@@ -11,7 +11,7 @@ from dateutil.tz import tzoffset, tzlocal
 import http.client
 from http.cookies import SimpleCookie
 from cgi import FieldStorage
-from omnitools import dt2yyyymmddhhmmss, HeadersDict, ApacheHeadersDict, str2html, encodeURIComponent, dt2rfc822gmt, rfc822gmt2dt
+from omnitools import dt2yyyymmddhhmmss, HeadersDict, ApacheHeadersDict, str2html, encodeURIComponent, dt2rfc822gmt, rfc822gmt2dt, b64d_and_utf8d
 
 
 _SERVER = dict(os.environ)
@@ -189,7 +189,7 @@ def _max_age(age: int = 0):
 
 
 def _cache_control():
-    set_header("Cache-Control", _max_age() + ", must-revalidate")
+    set_header("Cache-Control", _max_age()+", must-revalidate")
 
 
 def _should_read_from_cache_file():
@@ -272,12 +272,36 @@ def print_file(fo, buf_size: int = 32 * 1024, range = None):
     fo.close()
 
 
+def basic_authorization(credentials: list, realm: str):
+    def basic_auth():
+        return _basic_authorization(credentials, realm)
+
+    return basic_auth
+
+
+def _basic_authorization(credentials: list = None, realm: str = "Welcome! Please enter your username and password."):
+    if not credentials:
+        credentials = [{"admin": "admin"}]
+    if "Authorization" not in _HEADERS:
+        set_status(401)
+        set_header("WWW-Authenticate", "Basic realm={}, charset=\"UTF-8\"".format(realm))
+        return False
+    authorization = _HEADERS["Authorization"].split("Basic ")[-1]
+    u, p = b64d_and_utf8d(authorization).split(":")
+    if u in credentials and credentials[u] == p:
+        return True
+    set_status(401)
+    set_header("WWW-Authenticate", "Basic realm={}, charset=\"UTF-8\"".format(realm))
+    return False
+
+
 def execute(
         method: str = "get", cacheable: bool = False,
         cache_ctrl = _cache_control,
         cache_norm = _should_return_304,
         cache_strat = _should_read_from_cache_file,
         cache_store = _write_to_cache_file,
+        authentication = _basic_authorization,
         enable_tb: bool = True, traceback_kwargs: dict = None
     ):
     def wrapper(method_main):
@@ -293,12 +317,13 @@ def execute(
             pass
         def _execute():
             try:
-                if cacheable:
-                    cache_ctrl()
-                if cacheable and cache_norm():
-                    set_status(304)
-                elif not cacheable or (cacheable and not cache_strat()):
-                    method_main()
+                if authentication():
+                    if cacheable:
+                        cache_ctrl()
+                    if cacheable and cache_norm():
+                        set_status(304)
+                    elif not cacheable or (cacheable and not cache_strat()):
+                        method_main()
             except:
                 log(_traceback.format_exc(limit, chain))
                 __response["headers"].update(default_content_type)
